using System;
using System.IO;
using System.Text;
/// Template Processor
/// 
/// Implementiert die Template/Skinning Funktionalitten und sorgt
/// hauptschlich dafr, dass die Templates gecacht werden und zgig
/// an die Web-Applikation weitergegeben werden.
/// INFO: Caching noch nicht implementiert; fr sptere Versionen eingeplant
/// 
///  20.12.2004	-	initial Release										-	Daniel Kirstenpfad
///  27.12.2004	-	implementing Template Processor	-	Daniel Kirstenpfad
/// 
namespace VDRGUI
{
	#region TemplateData Structure
	/// <summary>
	/// Datenstruktur um Identifikationseigenschaften fr spezielle Sendungen/Sender/Anfragen zu bergeben
	/// Diese Datenstruktur wird notwendig, da der Template-Prozessor so generisch wie nur mglich
	/// aufgebaut sein soll - sodass man smtliche Keywords jederzeit anwenden knnte (solange gengen Informationen
	/// verfgbar sind: ich kann keine Details ber einen Sender anzeigen wenn ich garkeinen Sender-ID habe)
	/// Wird in Zukunft noch erweitert werden.
	/// </summary>
	public struct TemplateData
	{
		public string Channel_Identifier;
		public string Event_Identifier;
		public EPGData VDR_Daten;
	}
	#endregion

	/// <summary>
	/// Implementiert die Template/Skinning Funktionalitten und sorgt
	/// hauptschlich dafr, dass die Templates gecacht werden und zgig
	/// an die Web-Applikation weitergegeben werden.
	/// INFO: Caching noch nicht implementiert; fr sptere Versionen eingeplant
	/// </summary>
	public class TemplateProcessor
	{
		#region Objects and Data
		private TemplateData QueryData;
		#endregion

		#region Template Filenames
		// TODO: zum aktuellen Zeitpunkt sind die Template-Datei-Namen noch Hardcoded 
		private const string s_main_html_template = "template\\mainmenu.html.template";
		private const string s_whats_on_now = "template\\whatsonnow.html.template";
		private const string s_event_detailed_information = "template\\EventDetailedInformation.html";
		private const string s_root_request = "template\\index.html";
		#endregion

		#region Template Identifiers
		private const string s_tempalte_identifier_mainmenu_header = "main_header";													// Hauptmen-Titelzeile
		private const string s_template_identifier_mainmenu_horizontal = "main_menutable_horizontal";					// Hauptmen Horizontal
		private const string s_template_identifier_mainmenu_vertical = "main_menutable_vertical";							// Hauptmen Vertikal
		private const string s_template_identifier_whats_on_tv_header = "whatsontv_header";									// "Was luft gerade" Titelzeile
		private const string s_template_identifier_whats_on_tv = "whatsontv_content";													// "Was luft gerade" Inhalt
		private const string s_template_identifier_channel_episode_detailedinfo = "channel_detailinfo";						// detailierte Information ber eine Sendung
		private const string s_template_identifier_channel_listing  = "channel_listing";													// Listet alle Sendungen eines bestimmten Senders auf
		private const string s_template_identifier_channels_list = "channels_list";															// Listet alle Kanle auf
		private const string s_template_identifier_channel_name = "channel_name";														// Name eines spezifischen Senders
		private const string s_template_identifier_event_name = "event_name";																// Name der Sendung
		private const string s_template_identifier_event_short_description = "event_short_description";						// Kurzbeschreibung der Sendung
		private const string s_template_identifier_event_date_time_information = "event_date_time_information";	// Datums und Uhrzeitinformationen der Sendung
		private const string s_template_identifier_event_long_description = "event_long_description";						// detailierte Beschreibung der Sendung	
		#endregion

		#region Constructor / Deconstructor
		public TemplateProcessor()
		{
		}
		#endregion

		#region Specialised Template Processors
		/// <summary>
		/// Detailierte Informationen einer Sendung
		/// </summary>
		/// <param name="ParametricData">die Parameter-Datenstruktur</param>
		/// <returns>HTML Source des abgearbeiteten Templates</returns>
		public string EventDetailedInformationProcessor(TemplateData ParametricData)
		{
			return Processor(s_event_detailed_information,ParametricData);
		}
		/// <summary>
		/// Root Page
		/// </summary>
		/// <param name="ParametricData">die Parameter-Datenstruktur</param>
		/// <returns>HTML Source des abgearbeiteten Templates</returns>
		public string RootRequestProcessor(TemplateData ParametricData)
		{
			return Processor(s_root_request,ParametricData);
		}
		#endregion

		#region generic Template Processor
		/// <summary>
		///  generischer Template Prozessor fr die Keywords, alle Parameter werden aus der QueryData Struktur entnommen
		/// </summary>
		/// <param name="TemplateFile">Dateiname der Template-Datei</param>
		/// <param name="ParametricData">die Parameter-Datenstruktur</param>
		/// <returns>HTML Source des abgearbeiteten Templates</returns>
		public string Processor(string TemplateFile,TemplateData ParametricData)
		{
			StringBuilder HTMLOutput = new StringBuilder();
			QueryData = ParametricData;
			
			#region Startup Checks
			if (QueryData.VDR_Daten == null) return "No VDR Data available";
			if (!File.Exists(TemplateFile)) return "Template file not found";
			#endregion

			using (StreamReader TemplateStreamReader = new StreamReader(TemplateFile,System.Text.Encoding.Default))	
			{
				while (TemplateStreamReader.Peek() >= 0) 
				{
					string temporary = null;

					// eine Zeile aus dem Template in temporary einlesen
					temporary = TemplateStreamReader.ReadLine();

					#region parse and replace
					/// Vorgehensweise:
					///	1. von vorne durch den String laufen bis ein in %% eingefasstes Wort auftaucht, in diesem Falle Wort prfen
					///	wenn es ein Template-Keyword ist, dann entsprechenden HTML Quelltext anstelle der %keyword% einsetzen.
					///	dann von vorne beginnen bis kein Keyword mehr gefunden wird.
					bool found_keyword = false;

					/*	while(!found_keyword)
					{
						// % suchen
					}*/
					HTMLOutput.Append(temporary);
					#endregion
				}
			}


			return HTMLOutput.ToString();
		}
		#endregion
	}
}
