using System;
using System.Text;
using System.Data;

namespace VDRGUI
{
	/// <summary>
	/// Hier werden die eigentlichen HTML Quelltexte generiert
	/// </summary>
	public class WebApplicationHTMLGenerator
	{
		private EPGData VDR_Daten;
		private TemplateData TemplateProcessorData = new TemplateData();
		private TemplateProcessor TemplateEngine = new TemplateProcessor();

		/// <summary>
		/// Konstruktor der WebApplicationHTMLGenerator Klasse
		/// </summary>
		/// <param name="Data">die initialisierte EPG Daten-Klasse</param>
		public WebApplicationHTMLGenerator(EPGData Data)
		{
			VDR_Daten = Data;
			TemplateProcessorData.VDR_Daten = VDR_Daten;
		}

		#region WhatsOnTVNow
		/// <summary>
		/// Erzeugt eine "Was luft gerade" Tabelle mit allen Sendern
		/// </summary>
		/// <returns>"Was luft gerade" Tabelle als HTML Quelltext</returns>
		public string WhatsOnTvNow()
		{
			StringBuilder output = new StringBuilder();
			foreach (DataRow dr in VDR_Daten.channel.Rows)
			{
				output.Append("<b>Sender: "+dr["channel_name"].ToString()+"</b><br>");
				foreach (DataRow dr2 in dr.GetChildRows("channelepgData"))
				{
					DateTime tempDate = (DateTime)dr2["event_start"];

					output.Append(tempDate.ToString()+"<p>");

					output.Append(dr2["event_title"].ToString()+"<p>");
				}
			}
			return output.ToString();
		}
		#endregion

		#region Root Request
		/// <summary>
		/// Erzeugt die Startseite
		/// </summary>
		/// <returns>"Root Request /" als HTML Quelltext</returns>
		public string RootRequest()
		{
			StringBuilder output = new StringBuilder();
			// den Template-Prozessor mit der Template-Interpretierung beauftragen und Ergebniss abrufen
			output.Append(TemplateEngine.RootRequestProcessor(TemplateProcessorData));

			return output.ToString();
		}
		#endregion

		#region Event Popup
		/// <summary>
		/// erzeugt den HTML Quelltext mit den Detailinformationen einer speziellen Sendung
		/// </summary>
		/// <param name="event_string">eindeutige Identifikation der Sendung</param>
		/// <returns>HTML Quelltext mit Detailinformationen</returns>
		public string EventPopup(string event_string)
		{
			StringBuilder output = new StringBuilder();

			// den Template-Prozessor mit der Template-Interpretierung beauftragen und Ergebniss abrufen
			output.Append(TemplateEngine.EventDetailedInformationProcessor(TemplateProcessorData));

			return output.ToString();
		}
		#endregion


	}
}
