/// Dies ist ein Tool fr die Offline-Entwicklung von Applikationen fr den Linux VDR. Es
/// simuliert einen VDR Server Dummy mit zwar eingeschrnktem aber fr Tests ausreichendem
/// Funktionsumfang.
///
/// Es wird ein TCP-Server realisiert der auf einen hereinkommenden Client antwortet und
/// die Antworten auf die bekannten Befehle aus den *.response Dateien bezieht.
/// 
/// Um den Server um einen Befehl zu erweitern: siehe Region "Template Befehlsbehandlung"
/// 
///
/// Dieses Programm basiert auf dem MSDN TcpListener() Beispiel:
/// ms-help://MS.VSCC.2003/MS.MSDNQTR.2004OCT.1033/cpref/html/frlrfsystemnetsocketstcplistenerclasstopic.htm)
///
///	30.10.2004	-	Initial Release (Daniel Kirstenpfad)
///	31.10.2004	-	Erweiterung um beliebige Befehle; Antworten nicht hardcoded sondern in Form 
///					von Textdateien (*.response)
///					(Daniel Kirstenpfad
///	26.11.2004	-	Erweiterung um LSTT
///	

using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Text;

class simulatedVDRServer
{
	/// <summary>
	/// Liest alle Zeilen einer Textdatei in einen String ein, jede Zeile getrennt 
	/// durch Linefeed und Carriage-Return
	/// </summary>
	/// <param name="filename">Dateiname der Textdatei</param>
	/// <returns>String der den Inhalt der Textdatei enthlt</returns>
	public static string ReadResponseFile(string filename)
	{
		StringBuilder temporary = new StringBuilder();

		if (File.Exists(filename))
		{
			using (StreamReader TemplateReader = new StreamReader(filename,System.Text.Encoding.Default))		// bugfix 17.10.04 Matthias Eller
			{
				while (TemplateReader.Peek() >= 0) 
				{
					temporary.Append(TemplateReader.ReadLine());
					temporary.Append("\r\n");
				}
				TemplateReader.Close();
			}
		}
		else
		{
			Console.WriteLine(filename+" File not found");
		}
        return temporary.ToString();
	}
	/// <summary>
	/// Main-Methode
	/// </summary>
	public static void Main()
	{    
		try
		{			
			/// SVDRP Standardport 2001
			Int32 port = 2001;
			/// IP Adresse = Localhost; also nur lokal gebunden
			IPAddress localAddr = IPAddress.Parse("127.0.0.1");
      
			TcpListener server = new TcpListener(localAddr, port);

			// Auf Clients warten
			server.Start();
         
			// ein Byte einlesen; muss ein Byte-Array sein...
			Byte[] bytes = new Byte[1];
			StringBuilder data = new StringBuilder();

			Console.WriteLine("simulated VDR Server by Daniel Kirstenpfad");
			Console.WriteLine("This Server simulates an almost complete SVDRP VDR Server for offline testing\n");
			Console.WriteLine();
			Console.WriteLine("Server uses port "+port.ToString()+" on "+localAddr.ToString());
			Console.WriteLine("ctrl+c exits server");
			Console.WriteLine();

			// Listening Loop
			while(true) 
			{
				Console.Write("Waiting for a connection... ");
				bool quit = false;

				TcpClient client = server.AcceptTcpClient();            
				Console.WriteLine("Connected!");

				NetworkStream stream = client.GetStream();
				// Welcome-Message schicken
				string welcome_message = ReadResponseFile("WELCOME.response");
				stream.Write(System.Text.Encoding.ASCII.GetBytes(welcome_message),0,welcome_message.Length);

				while (!quit)
				{
					int i;
					data.Remove(0,data.Length);	// Incoming-Datenpuffer komplett entleeren
					bool incoming_wait_loop_quit = false;

					// Eingabe lesen
					while( ((i = stream.Read(bytes, 0, bytes.Length))!=0) & (!incoming_wait_loop_quit) )
					{   
						if (bytes[0] == '\r') incoming_wait_loop_quit = true;

						// Bytes in String umwandeln
						data.Append(System.Text.Encoding.ASCII.GetString(bytes, 0, i));
					}
					// Eingabe interpretieren

					#region Template Befehls-Behandlung
//					if (data.ToString().ToUpper().StartsWith("LSTR"))
//					{
//						/// LSTR Befehl
//						string Response = ReadResponseFile("LSTR.response");
//						stream.Write(System.Text.Encoding.ASCII.GetBytes(Response),0,Response.Length);						
//					}
					#endregion

					if (data.ToString().ToUpper().StartsWith("LSTR"))
					{
						/// LSTR Befehl
						string Response = ReadResponseFile("LSTR.response");
                        stream.Write(System.Text.Encoding.ASCII.GetBytes(Response),0,Response.Length);						
					}
					if (data.ToString().ToUpper().StartsWith("LSTT"))
					{
						/// LSTT Befehl
						string Response = ReadResponseFile("LSTT.response");
						stream.Write(System.Text.Encoding.ASCII.GetBytes(Response),0,Response.Length);						
					}
					if (data.ToString().ToUpper().StartsWith("STAT DISK"))
					{
						/// STAT DISK Befehl
						string Response = ReadResponseFile("STAT DISK.response");
						stream.Write(System.Text.Encoding.ASCII.GetBytes(Response),0,Response.Length);						
					}
					if (data.ToString().ToUpper().StartsWith("QUIT"))
					{
						/// QUIT Befehl
						/// Quit-Message schicken
						
						string quit_message = ReadResponseFile("QUIT.response");
						stream.Write(System.Text.Encoding.ASCII.GetBytes(quit_message),0,quit_message.Length);

						quit = true;
					}
				}
				// Verbindung beenden
				client.Close();
			}
		}
		catch(SocketException e)
		{
			Console.WriteLine("SocketException: {0}", e);
		}
  	}   
}
